@@ -8,12 +8,14 @@ from skfolio.model_selection._combinatorial import (
     CombinatorialPurgedCV,
     optimal_folds_number,
 )
+from skfolio.model_selection._multiple_randomized_cv import MultipleRandomizedCV
 from skfolio.model_selection._validation import cross_val_predict
 from skfolio.model_selection._walk_forward import WalkForward
 
 __all__ = [
     "BaseCombinatorialCV",
     "CombinatorialPurgedCV",
+    "MultipleRandomizedCV",
     "WalkForward",
     "cross_val_predict",
     "optimal_folds_number",
