@@ -5,6 +5,7 @@ from skfolio.optimization.cluster.hierarchical import (
     BaseHierarchicalOptimization,
     HierarchicalEqualRiskContribution,
     HierarchicalRiskParity,
+    SchurComplementary,
 )
 
 __all__ = [
@@ -12,4 +13,5 @@ __all__ = [
     "HierarchicalEqualRiskContribution",
     "HierarchicalRiskParity",
     "NestedClustersOptimization",
+    "SchurComplementary",
 ]
