@@ -7,9 +7,13 @@ from skfolio.optimization.cluster.hierarchical._herc import (
     HierarchicalEqualRiskContribution,
 )
 from skfolio.optimization.cluster.hierarchical._hrp import HierarchicalRiskParity
+from skfolio.optimization.cluster.hierarchical._schur import (
+    SchurComplementary,
+)
 
 __all__ = [
     "BaseHierarchicalOptimization",
     "HierarchicalEqualRiskContribution",
     "HierarchicalRiskParity",
+    "SchurComplementary",
 ]
