@@ -5,6 +5,8 @@
 # SPDX-License-Identifier: BSD-3-Clause
 # Weight constraints is a novel implementation, see docstring for more details.
 
+from __future__ import annotations
+
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
@@ -122,36 +124,35 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         :class:`~skfolio.cluster.HierarchicalClustering`.
 
     min_weights : float | dict[str, float] | array-like of shape (n_assets, ), default=0.0
-        Minimum assets weights (weights lower bounds). Negative weights are not allowed.
-        If a float is provided, it is applied to each asset.
-        If a dictionary is provided, its (key/value) pair must be the
-        (asset name/asset minium weight) and the input `X` of the `fit` methods must be
-        a DataFrame with the assets names in columns.
-        When using a dictionary, assets values that are not provided are assigned a
-        minimum weight of `0.0`. The default is 0.0 (no short selling).
+        Minimum assets weights (weights lower bounds). The default is 0.0 (no short
+        selling). Negative weights are not allowed. If a float is provided, it is
+        applied to each asset. `None` is equivalent to the default `0.0`. If a
+        dictionary is provided, its (key/value) pair must be the (asset name/asset
+        minimum weight) and the input `X` of the `fit` methods must be a DataFrame with
+        the asset names in columns. When using a dictionary, assets values that are not
+        provided are assigned the default  minimum weight of `0.0`.
 
         Example:
 
-           * min_weights = 0 --> long only portfolio (no short selling).
-           * min_weights = None --> no lower bound (same as `-np.Inf`).
-           * min_weights = {"SX5E": 0, "SPX": 0.1}
-           * min_weights = [0, 0.1]
+           * `min_weights = 0.0` --> long only portfolio (default).
+           * `min_weights = {"SX5E": 0.1, "SPX": 0.2}`
+           * `min_weights = [0.1, 0.2]`
 
     max_weights : float | dict[str, float] | array-like of shape (n_assets, ), default=1.0
-        Maximum assets weights (weights upper bounds). Weights above 1.0 are not
-        allowed. If a float is provided, it is applied to each asset.
-        If a dictionary is provided, its (key/value) pair must be the
-        (asset name/asset maximum weight) and the input `X` of the `fit` method must be
-        a DataFrame with the assets names in columns.
-        When using a dictionary, assets values that are not provided are assigned a
-        minimum weight of `1.0`. The default is 1.0 (each asset is below 100%).
+        Maximum assets weights (weights upper bounds). The default is 1.0 (each asset
+        is below 100%). Weights above 1.0 are not allowed. If a float is provided, it is
+        applied to each asset. `None` is equivalent to the default `1.0`. If a
+        dictionary is provided, its (key/value) pair must be the (asset name/asset
+        maximum weight) and the input `X` of the `fit` method must be a DataFrame with
+        the asset names in columns. When using a dictionary, assets values that are not
+        provided are assigned the default maximum weight of `1.0`.
 
         Example:
 
-           * max_weights = 0 --> no long position (short only portfolio).
-           * max_weights = 0.5 --> each weight must be below 50%.
-           * max_weights = {"SX5E": 1, "SPX": 0.25}
-           * max_weights = [1, 0.25]
+           * `max_weights = 1.0` --> each weight  must be below 100% (default).
+           * `max_weights = 0.5` --> each weight must be below 50%.
+           * `max_weights = {"SX5E": 0.8, "SPX": 0.9}`
+           * `max_weights = [0.8, 0.9]`
 
     transaction_costs : float | dict[str, float] | array-like of shape (n_assets, ), default=0.0
         Transaction costs of the assets. It is used to add linear transaction costs to
@@ -172,7 +173,7 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         If a float is provided, it is applied to each asset.
         If a dictionary is provided, its (key/value) pair must be the
         (asset name/asset cost) and the input `X` of the `fit` method must be a
-        DataFrame with the assets names in columns.
+        DataFrame with the asset names in columns.
         The default value is `0.0`.
 
         .. warning::
@@ -201,7 +202,7 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         If a float is provided, it is applied to each asset.
         If a dictionary is provided, its (key/value) pair must be the
         (asset name/asset fee) and the input `X` of the `fit` method must be a
-        DataFrame with the assets names in columns.
+        DataFrame with the asset names in columns.
         The default value is `0.0`.
 
         .. warning::
@@ -224,14 +225,10 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         portfolio total cost. If a float is provided, it is applied to each asset.
         If a dictionary is provided, its (key/value) pair must be the
         (asset name/asset previous weight) and the input `X` of the `fit` method must
-        be a DataFrame with the assets names in columns.
+        be a DataFrame with the asset names in columns.
         The default (`None`) means no previous weights.
-
-    portfolio_params :  dict, optional
-        Portfolio parameters passed to the portfolio evaluated by the `predict` and
-        `score` methods. If not provided, the `name`, `transaction_costs`,
-        `management_fees`, `previous_weights` and `risk_free_rate` are copied from the
-        optimization model and passed to the portfolio.
+        Additionally, when `fallback="previous_weights"`, failures will fall back to
+        these weights if provided.
 
     solver : str, default="CLARABEL"
         The solver used for the weights constraints optimization. The default is
@@ -246,6 +243,30 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         For more details about solver arguments, check the CVXPY documentation:
         https://www.cvxpy.org/tutorial/advanced/index.html#setting-solver-options
 
+    portfolio_params : dict, optional
+        Portfolio parameters forwarded to the resulting `Portfolio` in `predict`.
+        If not provided and if available on the estimator, the following
+        attributes are propagated to the portfolio by default: `name`,
+        `transaction_costs`, `management_fees`, `previous_weights` and `risk_free_rate`.
+
+    fallback : BaseOptimization | "previous_weights" | list[BaseOptimization | "previous_weights"], optional
+        Fallback estimator or a list of estimators to try, in order, when the primary
+        optimization raises during `fit`. Alternatively, use `"previous_weights"` 
+        (alone or in a list) to fall back to the estimator's `previous_weights`.
+        When a fallback succeeds, its fitted `weights_` are copied back to the primary 
+        estimator so that `fit` still returns the original instance. For traceability, 
+        `fallback_` stores the successful estimator (or the string `"previous_weights"`)
+         and `fallback_chain_` stores each attempt with the associated outcome.
+
+    raise_on_failure : bool, default=True
+        Controls error handling when fitting fails.
+        If True, any failure during `fit` is raised immediately, no `weights_` are
+        set and subsequent calls to `predict` will raise a `NotFittedError`.
+        If False, errors are not raised; instead, a warning is emitted, `weights_`
+        is set to `None` and subsequent calls to `predict` will return a
+        `FailedPortfolio`. When fallbacks are specified, this behavior applies only
+        after all fallbacks have been exhausted.
+
     Attributes
     ----------
     weights_ : ndarray of shape (n_assets,)
@@ -262,7 +283,28 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
 
     feature_names_in_ : ndarray of shape (`n_features_in_`,)
         Names of assets seen during `fit`. Defined only when `X`
-        has assets names that are all strings.
+        has asset names that are all strings.
+
+    fallback_ : BaseOptimization | "previous_weights" | None
+        The fallback estimator instance, or the string `"previous_weights"`, that
+        produced the final result. `None` if no fallback was used.
+
+    fallback_chain_ : list[tuple[str, str]] | None
+        Sequence describing the optimization fallback attempts. Each element is a
+        pair `(estimator_repr, outcome)` where `estimator_repr` is the string
+        representation of the primary estimator or a fallback (e.g. `"EqualWeighted()"`,
+        `"previous_weights"`), and `outcome` is `"success"` if that step produced
+        a valid solution, otherwise the stringified error message. For successful
+        fits without any fallback, this is `None`.
+
+    error_ : str | list[str] | None
+        Captured error message(s) when `fit` fails. For multi-portfolio outputs
+        (`weights_` is 2D), this is a list aligned with portfolios.
+
+    Notes
+    -----
+    All estimators should specify all parameters as explicit keyword arguments in
+    `__init__` (no `*args` or `**kwargs`), following scikit-learn conventions.
 
     References
     ----------
@@ -295,6 +337,8 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
         management_fees: skt.MultiInput = 0.0,
         previous_weights: skt.MultiInput | None = None,
         portfolio_params: dict | None = None,
+        fallback: skt.Fallback = None,
+        raise_on_failure: bool = True,
     ):
         super().__init__(
             risk_measure=risk_measure,
@@ -307,13 +351,15 @@ class HierarchicalEqualRiskContribution(BaseHierarchicalOptimization):
             management_fees=management_fees,
             previous_weights=previous_weights,
             portfolio_params=portfolio_params,
+            fallback=fallback,
+            raise_on_failure=raise_on_failure,
         )
         self.solver = solver
         self.solver_params = solver_params
 
     def fit(
         self, X: npt.ArrayLike, y: None = None, **fit_params
-    ) -> "HierarchicalEqualRiskContribution":
+    ) -> HierarchicalEqualRiskContribution:
         """Fit the Hierarchical Equal Risk Contribution estimator.
 
         Parameters
