@@ -12,9 +12,11 @@ from collections.abc import Iterator
 
 import numpy as np
 import pandas as pd
+import plotly.graph_objects as go
 
 import skfolio.typing as skt
 from skfolio.portfolio._base import BasePortfolio
+from skfolio.portfolio._failed_portfolio import FailedPortfolio
 from skfolio.portfolio._portfolio import Portfolio
 from skfolio.utils.tools import deduplicate_names
 
@@ -549,6 +551,30 @@ class MultiPeriodPortfolio(BasePortfolio):
         self.clear()
 
     # Classic property
+    @property
+    def failed_portfolios(self) -> list[FailedPortfolio]:
+        """Return the list of `FailedPortfolio` in the multi-period portfolio."""
+        return [x for x in self if isinstance(x, FailedPortfolio)]
+
+    @property
+    def fallback_portfolios(self) -> list[Portfolio]:
+        """
+        Return the list of portfolios in the multi-period portfolio that used a
+        fallback (i.e., have a non-None `fallback_chain`). This includes
+        `FailedPortfolio` instances when fallbacks were attempted.
+        """
+        return [x for x in self if getattr(x, "fallback_chain", None) is not None]
+
+    @property
+    def n_failed_portfolios(self) -> int:
+        """Number of `FailedPortfolio` in the multi-period portfolio."""
+        return len(self.failed_portfolios)
+
+    @property
+    def n_fallback_portfolios(self) -> int:
+        """Number of portfolios in the multi-period portfolio with a fallback."""
+        return len(self.fallback_portfolios)
+
     @property
     def assets(self) -> list:
         """List of assets names in each Portfolio."""
@@ -558,10 +584,30 @@ class MultiPeriodPortfolio(BasePortfolio):
     def composition(self) -> pd.DataFrame:
         """DataFrame of the Portfolio composition."""
         df = pd.concat([p.composition for p in self], axis=1)
-        df.fillna(0, inplace=True)
         df.columns = deduplicate_names(df.columns)
+        # Leave columns of only NaNs untouched
+        mask = ~df.isna().all(axis=0)
+        df.loc[:, mask] = df.loc[:, mask].fillna(0)
         return df
 
+    @property
+    def weights_dict(self) -> dict[str, dict[str, float]]:
+        """Dictionary mapping each Portfolio name to its asset weight allocation."""
+        names = deduplicate_names([ptf.name for ptf in self.portfolios])
+        return {
+            name: ptf.weights_dict
+            for name, ptf in zip(names, self.portfolios, strict=True)
+        }
+
+    @property
+    def previous_weights_dict(self) -> dict[str, dict[str, float]]:
+        """Dictionary mapping Portfolio name to its previous asset weight allocation."""
+        names = deduplicate_names([ptf.name for ptf in self.portfolios])
+        return {
+            name: ptf.previous_weights_dict
+            for name, ptf in zip(names, self.portfolios, strict=True)
+        }
+
     @property
     def weights_per_observation(self) -> pd.DataFrame:
         """DataFrame of the Portfolio weights per observation."""
@@ -603,8 +649,10 @@ class MultiPeriodPortfolio(BasePortfolio):
         if not to_df:
             return contributions
         df = pd.concat(contributions, axis=1)
-        df.fillna(0, inplace=True)
         df.columns = deduplicate_names(df.columns)
+        # Leave columns of only NaNs untouched
+        mask = ~df.isna().all(axis=0)
+        df.loc[:, mask] = df.loc[:, mask].fillna(0)
         return df
 
     def summary(self, formatted: bool = True) -> pd.Series:
@@ -622,13 +670,22 @@ class MultiPeriodPortfolio(BasePortfolio):
             Portfolio summary of all its measures.
         """
         df = super().summary(formatted=formatted)
-        portfolios_number = len(self)
         avg_assets_per_portfolio = np.mean([p.n_assets for p in self])
+        n_portfolios = len(self)
+        n_failed_portfolios = self.n_failed_portfolios
+        n_fallback_portfolios = self.n_fallback_portfolios
+
         if formatted:
-            portfolios_number = str(int(portfolios_number))
             avg_assets_per_portfolio = f"{avg_assets_per_portfolio:0.1f}"
-        df["Portfolios Number"] = portfolios_number
+            n_portfolios = str(int(n_portfolios))
+            n_failed_portfolios = str(n_failed_portfolios)
+            n_fallback_portfolios = str(n_fallback_portfolios)
+
         df["Avg nb of Assets per Portfolio"] = avg_assets_per_portfolio
+        df["Number of Portfolios"] = n_portfolios
+        df["Number of Failed Portfolios"] = n_failed_portfolios
+        df["Number of Fallback Portfolios"] = n_fallback_portfolios
+
         return df
 
     # Public methods
@@ -651,7 +708,7 @@ class MultiPeriodPortfolio(BasePortfolio):
         self._loaded = False
         self._portfolios.append(portfolio)
         if len(self.observations) == 0:
-            # We don"t concatenate an empty array as we cannot know the dtype before.
+            # We don't concatenate an empty array as we cannot know the dtype before.
             self.observations = portfolio.observations
             self.returns = portfolio.returns
         else:
@@ -661,3 +718,50 @@ class MultiPeriodPortfolio(BasePortfolio):
             self.returns = np.concatenate([self.returns, portfolio.returns], axis=0)
         self._loaded = True
         self.clear()
+
+    def plot_weights_per_observation(self):
+        """Plot portfolio weights per observation as a stacked-area chart.
+
+        This shows the composition of the portfolio over time, with each asset's weight
+        stacked to illustrate how allocations shift.
+
+        Returns
+        -------
+        plot : Figure
+            Returns the plot Figure object.
+        """
+        df = self.weights_per_observation
+
+        fig = go.Figure()
+
+        for asset in df.columns:
+            fig.add_trace(
+                go.Scatter(
+                    x=df.index,
+                    y=df[asset],
+                    mode="lines",
+                    name=asset,
+                    stackgroup="one",  # stack all series
+                    line=dict(width=0.5),
+                    hovertemplate=(
+                        "%{x|%Y-%m-%d}<br>"  # date
+                        f"{asset}: "  # asset name
+                        "%{y:.2%}"  # two-decimals percent
+                        "<extra></extra>"
+                    ),
+                )
+            )
+        fig.update_layout(
+            title="Weight allocation over time",
+            xaxis_title="Date",
+            yaxis_title="Weight (%)",
+            legend_title_text="Assets",
+        )
+
+        fig.update_yaxes(
+            tickformat=".0%",
+            zeroline=True,
+            zerolinecolor="gray",
+        )
+
+        return fig
