@@ -1,11 +1,14 @@
 """Tools module."""
 
+import math
+import random
 import warnings
 
 # Copyright (c) 2023
 # Author: Hugo Delatte <delatte.hugo@gmail.com>
 # SPDX-License-Identifier: BSD-3-Clause
 # Implementation derived from:
+# Precise, Copyright (c) 2021, Peter Cotton.
 # Riskfolio-Lib, Copyright (c) 2020-2023, Dany Cajas, Licensed under BSD 3 clause.
 # Statsmodels, Copyright (C) 2006, Jonathan E. Taylor, Licensed under BSD 3 clause.
 from enum import auto
@@ -26,17 +29,22 @@ __all__ = [
     "assert_is_distance",
     "assert_is_square",
     "assert_is_symmetric",
+    "combination_by_index",
     "commutation_matrix",
     "compute_optimal_n_clusters",
     "corr_to_cov",
     "cov_nearest",
     "cov_to_corr",
+    "inverse_multiply",
     "is_cholesky_dec",
     "minimize_relative_weight_deviation",
+    "multiply_by_inverse",
     "n_bins_freedman",
     "n_bins_knuth",
     "rand_weights",
     "rand_weights_dirichlet",
+    "sample_unique_subsets",
+    "symmetric_step_up_matrix",
 ]
 
 
@@ -142,8 +150,8 @@ def rand_weights_dirichlet(n: int) -> np.array:
     return np.random.dirichlet(np.ones(n))
 
 
-def rand_weights(n: int, zeros: int = 0) -> np.array:
-    """Produces n random weights that sum to one from an uniform distribution
+def rand_weights(n: int, zeros: int = 0, seed: int | None = None) -> np.ndarray:
+    """Produces n random weights that sum to one from a uniform distribution
     (non-uniform distribution over a simplex).
 
     Parameters
@@ -154,16 +162,21 @@ def rand_weights(n: int, zeros: int = 0) -> np.array:
     zeros : int, default=0
         The number of weights to randomly set to zeros.
 
+    seed : int or None, default=None
+        Seed for reproducibility. If None, use an unseeded generator.
+
     Returns
     -------
     weights : ndarray of shape (n, )
         The vector of weights.
     """
-    k = np.random.rand(n)
+    rng = np.random.default_rng(seed)
+
+    k = rng.random(n)
     if zeros > 0:
-        zeros_idx = np.random.choice(n, zeros, replace=False)
+        zeros_idx = rng.choice(n, zeros, replace=False)
         k[zeros_idx] = 0
-    return k / sum(k)
+    return k / k.sum()
 
 
 def is_cholesky_dec(x: np.ndarray) -> bool:
@@ -323,7 +336,7 @@ def cov_nearest(
     matrix using the initial standard deviation.
 
     Cholesky decomposition can fail for symmetric positive definite (SPD) matrix due
-    to floating point error and inversely, Cholesky decomposition can success for
+    to floating point error and inversely, Cholesky decomposition can succeed for
     non-SPD matrix. Therefore, we need to test for both. We always start by testing
     for Cholesky decomposition which is significantly faster than checking for positive
     eigenvalues.
@@ -451,7 +464,6 @@ def compute_optimal_n_clusters(distance: np.ndarray, linkage_matrix: np.ndarray)
 
     with :math:`d(u,v)` the distance between u and v.
 
-
     Parameters
     ----------
     distance : ndarray of shape (n, n)
@@ -575,3 +587,215 @@ def minimize_relative_weight_deviation(
         ) from None
 
     return w.value
+
+
+def combination_by_index(idx: int, n: int, k: int) -> np.ndarray:
+    """
+    Retrieve the k-combination at a given lexicographic position without enumerating
+    all combinations.
+
+    This function implements the *unranking* algorithm (also known as the combinatorial
+    number system or "combinadic") to retrieve the specific k-combination corresponding
+    to a given lexicographic `idx` without generating all C(n, k) possible subsets.
+
+    Given a universe of size `n`, there are M = C(n, k) possible subsets of size k.
+    This function returns the subset corresponding to the `idx` in lex order.
+
+    This approach is crucial when M = C(n, k) is too large to generate or store all
+    combinations, and you need to draw random subsets uniformly (sampling k=5 from n=100
+    gives M ≈ 7.5e7).
+
+    Time complexity: O(k)
+    Space complexity: O(k)
+
+    Parameters
+    ----------
+    idx : int
+        Index (rank) of the desired combination in lex order. Must satisfy
+        0 <= idx < C(n, k).
+
+    n : int
+        Size of the universe.
+
+    k : int
+        Size of each combination (0 <= k <= n).
+
+    Returns
+    -------
+    combination : ndarray of shape (k,)
+        1D integer array of length k containing the sorted k-combination.
+
+    Raises
+    ------
+    ValueError
+        If parameters are out of valid range.
+
+    References
+    ----------
+    ..[1] "The Art of Computer Programming", Vol. 4A: Combinatorial Algorithms,
+      Section 7.2.1.3. Knuth, D. E. (1998).
+    """
+    total = math.comb(n, k)
+    if idx < 0 or idx >= total:
+        raise ValueError(
+            f"Index {idx} out of range for C({n},{k})={total} combinations."
+        )
+
+    combination = np.empty(k, dtype=int)
+    remaining_rank = idx
+    next_element = 0
+
+    for pos in range(k):
+        remaining_slots = k - pos
+        x = next_element
+        block_size = math.comb(n - x - 1, remaining_slots - 1)
+        while block_size <= remaining_rank:
+            remaining_rank -= block_size
+            x += 1
+            block_size = math.comb(n - x - 1, remaining_slots - 1)
+        combination[pos] = x
+        next_element = x + 1
+
+    return combination
+
+
+def sample_unique_subsets(
+    n: int, k: int, n_subsets: int, random_state: int | None = None
+) -> np.ndarray:
+    """
+    Generate unique k-element subsets from a universe of size n using combinatorial
+    unranking.
+
+    Each subset is drawn without replacement (elements within subset are distinct) and
+    no subset is repeated across draws. Ranks are sampled uniformly without replacement
+    over [0, C(n, k)).
+
+    Time complexity: O(n_subsets * k)
+    Space complexity: O(n_subsets * k)
+
+    Parameters
+    ----------
+    n : int
+        Universe size.
+
+    k : int
+        Subset size (0 <= k <= n).
+
+    n_subsets : int
+        Number of distinct subsets to generate (0 <= n_subsets <= C(n, k)).
+
+    random_state : int, RandomState instance or None, default=None
+        Seed or random state to ensure reproducibility.
+
+    Returns
+    -------
+    subsets : ndarray of shape (n_subsets, k)
+        2D integer array of shape (n_subsets, k) where each row is a sorted
+        k-combination.
+
+    Raises
+    ------
+    ValueError
+        If any parameters are out of valid ranges.
+    """
+    if n < 0:
+        raise ValueError(f"n must be non-negative, got {n}.")
+    if k < 0 or k > n:
+        raise ValueError(f"k={k} must satisfy 0 <= k <= n={n}.")
+
+    total = math.comb(n, k)
+    if n_subsets < 0 or n_subsets > total:
+        raise ValueError(
+            f"n_subsets={n_subsets} must satisfy 0 <= n_subsets <= C({n},{k})={total}."
+        )
+
+    rng = random.Random(random_state)
+    ranks = rng.sample(range(total), k=n_subsets)
+    # random.sample has a special-case for range objects that avoids building a list of
+    # length M=C(n,k) and runs in O(n_subsets) time and space as opposed to
+    # `choice(total, size=n_subsets, replace=False)` which run in O(M) space and raises
+    # ArrayMemoryError for very big M.
+    subsets = np.empty((n_subsets, k), dtype=int)
+    for i, rank in enumerate(ranks):
+        subsets[i, :] = combination_by_index(rank, n, k)
+
+    return subsets
+
+
+def inverse_multiply(a: np.ndarray, b: np.ndarray) -> np.ndarray:
+    """Multiply the inverse of matrix a by matrix b.
+    We use np.linalg.solve as it tends to produce more accurate results than
+    np.linalg.inv.
+
+    Parameters
+    ----------
+    a : ndarray of shape (n, n)
+        Square matrix.
+
+    b : ndarray of shape (n, m)
+        Matrix.
+
+    Returns
+    -------
+    m : ndarray of shape (n, m)
+        The inverse of matrix a multiplied by matrix b.
+    """
+    assert_is_square(a)
+    if a.shape[1] != b.shape[0]:
+        raise ValueError("Wrong dimension")
+    return np.linalg.solve(a, b)
+
+
+def multiply_by_inverse(a: np.ndarray, b: np.ndarray) -> np.ndarray:
+    """Multiply matrix a by the inverse of matrix b.
+    We use np.linalg.solve as it tends to produce more accurate results than
+    np.linalg.inv.
+
+    Parameters
+    ----------
+    a : ndarray of shape (n, m)
+        Matrix.
+
+    b : ndarray of shape (n, n)
+        Square matrix.
+
+    Returns
+    -------
+    m : ndarray of shape (n, m)
+        The matrix a multiplied by the inverse of matrix b.
+    """
+    return inverse_multiply(b.T, a.T).T
+
+
+def symmetric_step_up_matrix(n1: int, n2: int) -> np.ndarray:
+    """Compute the Symmetric step-up matrix M such that `M @ np.ones(n2) = np.ones(n1)`.
+
+    Parameters
+    ----------
+    n1 : int
+        First dimension.
+
+    n2 : int
+        Second dimension.
+
+    Returns
+    -------
+    m : ndarray of shape (n1, n2)
+        The Symmetric step-up matrix.
+    """
+    assert abs(n1 - n2) <= 1
+
+    if n1 == n2:
+        return np.eye(n1)
+
+    if n1 < n2:
+        return symmetric_step_up_matrix(n2, n1).T * n1 / n2
+
+    m = np.zeros((n1, n2))
+    j_row = np.ones((1, n2)) / n2
+    e = np.eye(n2)
+    for j in range(n1):
+        mj = np.concatenate([e[:j], j_row, e[j:]], axis=0)
+        m += mj / n1
+
+    return m
